rm(list=ls())
library(ggplot2)
LandisOutputDir <- "G:\\Yan\\LANDIS\\CBFA\\ALPAC\\2020-02-06"

studyAreas <- c("ALPAC_OnlyDynamicArea")
timestep <- 10
StartYear <- 2000

FixedYAxisStacked <- TRUE
ylimStacked <- c(0,150)

ManualColorSpp <- TRUE

ManualColorSpp.df <- as.data.frame(rbind(

		cbind("LARI.LAR", "yellowgreen", "Larch"),
		cbind("PICE.GLA", "green2", "White Spruce"),
		cbind("PICE.MAR", "seagreen1", "Black Spruce"),
		cbind("PINU.BAN", "green4", "Jack Pine"),


		cbind("BETU.PAP", "orange", "White Birch"),
		cbind("POPU.BAL", "khaki2", "Balsam Poplar"),
		cbind("POPU.TRE", "khaki3", "Trembling Aspen")#,



		))

OrderSpecies <- TRUE # order according to above df


NameOfRun <- "OnlyDynamicArea"


selectYears <- c(1, seq(10,200,10))

ByZoomArea <- TRUE
ZoomIsMultiClass <- FALSE
UseBiomassPerLandtype <- FALSE

#Eliminate some landtypes?
EliminateLandtypes <- TRUE
LandtypesToKeep <- "_0"

### 'vegCodes' is the species master list
### It indicates which species to look in picus output folders
### 'ecoNames' gives the ecozones full names
require(RCurl)
readURL <- "https://raw.githubusercontent.com/dcyr/LANDIS-II_IA_generalUseFiles/master/"
vegCodes <- read.csv(text = getURL(paste(readURL, "vegCodes.csv", sep="/"),.opts = list(ssl.verifypeer = FALSE)))
ecoNames <- read.csv(text = getURL(paste(readURL, "ecoNames.csv", sep="/"),.opts = list(ssl.verifypeer = FALSE)))


TreatmentsToKeep <- c("BaselineFireDrought_0_mortHarvest", "BaselineFireDrought_3_mortHarvest", "BaselineFireDrought_6_mortHarvest",
				"GrowthProjectedFireDrought_0_mortHarvest", "GrowthProjectedFireDrought_3_mortHarvest", "GrowthProjectedFireDrought_6_mortHarvest")


### sometimes useful to change that...
options(scipen=0)
###

require(ggplot2)
require(RColorBrewer)
require(reshape2)
require(dplyr)
require(doBy)


a <- studyAreas


### setting working directory in the folder that is parent of the one of Landis Outputs
setwd(LandisOutputDir)
setwd("..")
wwd <- paste(getwd(), Sys.Date(), sep="/")
dir.create(wwd)
setwd(wwd)
rm(wwd)
readDir <- getwd()


#ecoName <- ecoNames[which(ecoNames$code==a),"name"]

if(ByZoomArea){

	biomassToUse <- get(load(paste(LandisOutputDir, "/biomassByZoomAreaByEcodistrict_", a, ".RData", sep="")))## loads 'biomassToUse' data frame
	} else {
	biomassToUse <- get(load(paste(LandisOutputDir, "/biomassPerEcodistrict_", a, ".RData", sep="")))## loads 'biomassToUse' data frame
	}

if(UseBiomassPerLandtype){
	biomassToUse <- get(load(paste(LandisOutputDir, "/biomassPerLandtype_", a, ".RData", sep="")))## loads 'biomassToUse' data frame
	}
if(TreatmentsToKeep!="all"){
	biomassToUse <- subset(biomassToUse, Treatment %in% TreatmentsToKeep)
	}

# if landtypes to eliminate

if(EliminateLandtypes){
	LinesToKeep <- grep(LandtypesToKeep, biomassToUse$Landtype)
	biomassToUse <- biomassToUse[LinesToKeep,]
	}

names(biomassToUse)[names(biomassToUse) == "areaEcodistrictTotal_ha"] <- "areaTotal_ha"
names(biomassToUse)[names(biomassToUse) == "areaLandtypeTotal_ha"] <- "areaTotal_ha"
names(biomassToUse)[names(biomassToUse) == "biomassEcodistrictTotal_Tons"] <- "biomassTotal_Tons"
names(biomassToUse)[names(biomassToUse) == "biomassLandtypeTotal_Tons"] <- "biomassTotal_Tons"


if(ZoomIsMultiClass){

	ClassesNamesdf <- as.data.frame(do.call(rbind, ClassesNames))
	ClassesNamesdf <- data.frame(row.names(ClassesNamesdf), ClassesNamesdf)
	names(ClassesNamesdf) <- c("Classes", "ClassesNames")

	ClassesNamesdf$Classes <- as.numeric(as.character(ClassesNamesdf$Classes))

	totalArea <- NULL
	for(i in unique(biomassToUse$Classes)){
		data.group <- subset(biomassToUse, Classes==i)
		data.group <- subset(data.group, SimArea == unique(data.group$SimArea)[1] & Scenario == unique(data.group$Scenario)[1] & Treatment == unique(data.group$Treatment)[1] & Replicate == unique(data.group$Replicate)[1] & Time == unique(data.group$Time)[1] & Species == unique(data.group$Species)[1])
		totalAreag <- data.frame(i,sum(data.group$areaTotal_ha))
		totalArea <- rbind(totalArea, totalAreag)
		}
	names(totalArea) <- c("Classes", "areaTotal_ha")

	} else {
	totalArea <- sum(unique(biomassToUse$areaTotal_ha))

	}

## renaming levels for making facets labels more explicit
biomassToUse$Ecodistrict <-  as.factor(biomassToUse$Ecodistrict)

if(UseBiomassPerLandtype){
	biomassToUse$Landtype<-  as.factor(biomassToUse$Landtype)
	}

treatments <- levels(biomassToUse$Treatment)

## extract fire regime from 'treatment'
fire <- ifelse(biomassToUse$Treatment %in% treatments[grep("ProjectedFire", treatments)],
                      "Fire regime: projected", "Fire regime: baseline")
fire <- factor(fire, levels = c("Fire regime: baseline", "Fire regime: projected"))

if(length(grep("Harvest", biomassToUse$Treatment)) == 0){
	harvest <- biomassToUse$Treatment
	} else { 

	## extract harvest treatment from 'treatment'
	harvest <- gsub("Budworm", "", biomassToUse$Treatment)
	harvest <- gsub("Growth", "", harvest)
	harvest <- gsub("ProjectedFire", "", harvest)
	harvest <- gsub("BaselineFire", "", harvest)
	harvest <- gsub("Harvest", "", harvest)
	harvest <- ifelse(harvest == "", "No", harvest)
	}

##For ALPAC
harvestLabels <- c("Drought_0_mort" = "No Harvesting",
              "Drought_3_mort" = "Baseline Harvesting",
                 "Drought_6_mort" = "High Harvesting")



##############################################################
##############################################################

names(ManualColorSpp.df) <- c("Species", "cols", "NiceSpecies")
ManualColorSpp.df$Species <- as.character(ManualColorSpp.df$Species)  
ManualColorSpp.df$cols <- as.character(ManualColorSpp.df$cols) 
ManualColorSpp.df$NiceSpecies<- as.character(ManualColorSpp.df$NiceSpecies)  






harvest <- factor(harvestLabels[harvest], levels = harvestLabels)


## putting everything back in the data.frame
biomassToUse[, "fire"] <- fire
biomassToUse[, "harvest"] <- harvest

## a little more cleaning up..
## species
biomassToUse$Species <- as.character(biomassToUse$Species)
# removing 'TotalBiomass', that Yan forgot to remove from one df somehow... ;-)
biomassToUse <-biomassToUse[biomassToUse$Species!="TotalBiomass",]
biomassToUse <- droplevels(biomassToUse)
# retreiving colors for species on the spp master list
spp <- unique(biomassToUse$Species)
if(ManualColorSpp){
	ManualColorSpp.df <- subset(ManualColorSpp.df, Species %in% spp)
	cols <- ManualColorSpp.df$cols

	} else {
	
	cols <- as.character(vegCodes[match(spp, vegCodes$LandisCode), "color"])
	}

# removing the annowing '.ALL' on 'THUJ.SPP'
biomassToUse[which(biomassToUse$Species=="THUJ.SPP.ALL"),"Species"] <- "THUJ.SPP"
if(ManualColorSpp){
	ManualColorSpp.df[which(ManualColorSpp.df$Species=="THUJ.SPP.ALL"),"Species"] <- "THUJ.SPP"
	}

biomassToUse$Species <- factor(biomassToUse$Species)


### calculating averaged values (all replicates) By now, I should really use 'dplyr' for that stuff

if(UseBiomassPerLandtype){

	if(ZoomIsMultiClass){

		df1 <- aggregate(biomassToUse$biomassTotal_Tons,
                 by=biomassToUse[, c("SimArea", "Replicate",
                                              "Scenario",
                                              "harvest",
                                              "fire",
                                              "Time",
                                              "Species",
							    "Classes",
                                              "Ecodistrict")],  FUN=sum, na.rm=TRUE)
		} else {




		df1 <- aggregate(biomassToUse$biomassTotal_Tons,
                 by=biomassToUse[, c("SimArea", "Replicate",
                                              "Scenario",
                                              "harvest",
                                              "fire",
                                              "Time",
                                              "Species",
                                              "Ecodistrict")],  FUN=sum, na.rm=TRUE)

		}
 
	} else {
	
	df1 <- biomassToUse
	}

names(df1)[names(df1) == "x"] <- "biomassTotal_Tons"

if(ZoomIsMultiClass){

	df2 <- aggregate(df1$biomassTotal_Tons,
                 by=df1[, c("SimArea",
                                              "Scenario",
                                              "harvest",
                                              "fire",
                                              "Time",
                                              "Species",
							    "Classes",
                                              "Ecodistrict")],  FUN=mean, na.rm=TRUE)
	} else {




	df2 <- aggregate(df1$biomassTotal_Tons,
                 by=df1[, c("SimArea",
                                              "Scenario",
                                              "harvest",
                                              "fire",
                                              "Time",
                                              "Species",
                                              "Ecodistrict")],  FUN=mean, na.rm=TRUE)

	}


### defining breaks
if(max(df2$Time)==100) {
    breaks <- c(2010, 2040, 2070, 2100)
} else {
    breaks <- seq(from=StartYear, to=max(StartYear+df2$Time), by=100)
}


## sometimes useful to filter out some scenarios / treatments
df <- df2 #%>%
   #filter(grepl(f, fire) | Scenario == "baseline")


df <- droplevels(df)


## defining fig size based on number of facets
pWidth <- (length(unique(df$harvest))+3)*480#(length(unique(df$harvest))+3)*480
pHeigth <- (length(levels(df$Scenario))+2)*500

if(!is.null(selectYears)){
	df <- subset(df, Time %in%  selectYears)
	}

if(ManualColorSpp){
	df <- merge(df, ManualColorSpp.df)
	df$Species <- factor(df$Species, levels = ManualColorSpp.df$Species)
	df$NiceSpecies <- factor(df$NiceSpecies , levels = ManualColorSpp.df$NiceSpecies )
	} else

	df$NiceSpecies <- df$Species
	}

for (i in c("stack", "fill", "line")) { # i<- "stack" {

	if(ZoomIsMultiClass){
		for(z in unique(df$Classes)){
			dfZ <- subset(df, Classes == z)
			ClassesNamesToGraph <- ClassesNamesdf$ClassesNames[which(ClassesNamesdf$Classes == z)]
			totalAreaToGraph <- totalArea$areaTotal_ha[which(totalArea$Classes == z)]
		

			g3 <- ggplot(dfZ, aes(x=Time+StartYear, y=x/totalAreaToGraph, fill=NiceSpecies, colour = Species))
			if (i == "stack") {
				g3 <- g3 +
					stat_summary(fun.y="sum", geom="area", col=NA, position = position_stack(reverse = TRUE)) +
					scale_fill_manual(values=cols) +
					labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n", ClassesNamesToGraph, "\n"),
						subtitle = "Cumulative abundance")
				if(FixedYAxisStacked){

					g3 <- g3 + ylim(ylimStacked)
					}


				}
			if (i == "fill") {
				g3 <- g3 +
					stat_summary(fun.y="sum", geom="area", col=NA, position = position_fill(reverse = TRUE)) +
					scale_fill_manual(values=cols) +
					labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n", ClassesNamesToGraph, "\n"),
						subtitle = "Relative abundance")
				}
			if (i == "line") {
				g3 <- g3 +
					stat_summary(fun.y="sum", geom="line", size=3) +
					scale_colour_manual(values=cols) +
					labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n", ClassesNamesToGraph, "\n"),
						subtitle = "Absolute abundance")
				}
			g3 <- g3 +
				facet_grid( Scenario ~ harvest) +
				#scale_fill_manual(values=cols) +
				guides(fill = guide_legend(reverse = T),
					colour = guide_legend(reverse = T)) +
				labs(#title = paste0("Evolution of landscape composition\n ", ecoName, " - ",s, "\n", ClassesNamesToGraph, "\n"),
					y = "Mean aboveground biomass\n(t/ha)\n")
    
			png(filename=paste0("biomassSpp",i,"_", a, "_", ClassesNamesToGraph, "_", NameOfRun, ".png"),
				width = pWidth, height = 2400, pointsize=72)
			print(g3 + theme_grey(base_size=72) +
			scale_x_continuous(breaks = breaks) +
			theme(axis.title.x = element_blank(),
                    axis.text.x = element_text(size=44, angle = 45, hjust = 1),
                    axis.text.y = element_text(size=48),
                    strip.text.x = element_text(size=40),
                    strip.text.y = element_text(size=60)))
			dev.off()
			}

		} else {

		g3 <- ggplot(df, aes(x=Time+StartYear, y=x/totalArea, fill=NiceSpecies, colour = Species))
		if (i == "stack") {
			g3 <- g3 +
				stat_summary(fun.y="sum", geom="area", col=NA, position = position_stack(reverse = TRUE)) +
				scale_fill_manual(values=cols) +
				labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n"),
					subtitle = "Cumulative abundance")

			if(FixedYAxisStacked){

				g3 <- g3 + ylim(ylimStacked)
				}

			}

		if (i == "fill") {
			g3 <- g3 +
				stat_summary(fun.y="sum", geom="area", col=NA, position = position_fill(reverse = TRUE)) +
				scale_fill_manual(values=cols) +
				labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n"),
					subtitle = "Relative abundance")
			}
		if (i == "line") {
			g3 <- g3 +
				stat_summary(fun.y="sum", geom="line", size=3) +
				scale_colour_manual(values=cols) +
				labs(title = paste0("Evolution of landscape composition\n", studyAreas, "\n"),
					subtitle = "Absolute abundance")
			}
		g3 <- g3 +
			facet_grid( Scenario ~ harvest) +
			#scale_fill_manual(values=cols) +
			guides(fill = guide_legend(reverse = T),
				colour = guide_legend(reverse = T)) +
			labs(#title = paste0("Evolution of landscape composition\n ", ecoName, " - ",s, "\n"),
				y = "Mean aboveground biomass\n(t/ha)\n")


    
		png(filename=paste0("biomassSpp",i,"_", a, "_", NameOfRun, ".png"),
		width = pWidth, height = 2400, pointsize=72)
		print(g3 + theme_grey(base_size=72) +
			scale_x_continuous(breaks = breaks) +
			theme(axis.title.x = element_blank(),
                    axis.text.x = element_text(size=44, angle = 45, hjust = 1),
                    axis.text.y = element_text(size=48),
                    strip.text.x = element_text(size=40),
                    strip.text.y = element_text(size=60)))
		dev.off()
		}

	}


